#
# WiKicker
#
#  CGI.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: AbstractController.pm,v 1.8 2004/01/29 14:32:34 naney Exp $
#
#

package WiKicker::CGI::AbstractController;

use 5.005;
use strict;
use CGI qw/-no_xhtml/;
use Time::HiRes qw(gettimeofday);
use WiKicker::CGI::Response;
use vars qw($Run_count);

BEGIN { $Run_count = 0; }

sub new {
  my ($class, %options) = @_;

  my $start_time = [ gettimeofday ];

  my $self = bless { start_time => $start_time }, $class;
  $self->{options} = \%options;

  return $self;
}

sub run {
  my $self = shift;

  $Run_count++;
  $CGI::XHTML = 0;
  my $query = new CGI;

  my $configuration = $self->_get_configuration($query);
  my $environment = $self->_get_environment($configuration);
  my $request = $self->_get_request($configuration, $query);
  $request->attribute('run_count', $Run_count);
  $request->environment($environment);
  return if $self->_run_hook($request);
  my $response = $self->_get_response;
  my $forward;
  {
    my $page = $self->_create_page($request);
    eval {
      $forward = $page->action($response);
    };
    if ($@) {
      my $exception = $@;
      if ($exception =~ /^Unknown action/) {
        die $exception;
      }
      else {
        die $exception;
      }
    }
    #IFDEBUG: WiKicker::cycle_check($page);
  }

  my $cookie_domain = $request->configuration->cookie_domain;
  my $cookie_path = $request->configuration->cookie_path;
      
  $response->add_cookie(WiKicker::CGI::Cookie::make_user_cookie($request->user, $cookie_domain, $cookie_path));
  
  $self->_view($forward, $request, $response);

  $self->_log($request);

  #IFDEBUG: WiKicker::cycle_check($request);
  #IFDEBUG: WiKicker::total_size('$request', $request);
  #IFDEBUG: WiKicker::cycle_check($response);
  #IFDEBUG: WiKicker::total_size('$response', $response);
}

sub _view {
  my ($self, $forward, $request, $response) = @_;
  my $view_class = $request->configuration->view_class($forward, $request->format);
  eval "use $view_class";
  if ($@) {
    die "Can't load view class '$view_class': $@";
  }

  eval {
    $view_class->new($request)->view($request, $response, $self->{start_time});
  };
  if ($@) {
    die "View error: $@";
  }

  $response->flush;
}

sub _log {
  my ($self, $request) = @_;

  # for override;
}

sub _get_response {
  my $self = shift;

  return WiKicker::CGI::Response->new;
}

1;
