#
# WiKicker
#
#  HTML.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HTML.pm,v 1.8 2004/01/17 08:42:15 naney Exp $
#
#

package WiKicker::CGI::View::HTML;

use 5.005;
use strict;
use CGI qw/-no_xhtml/;
use WiKicker::CGI::View;
use WiKicker::Time;

use vars qw(@ISA);

BEGIN {
  @ISA = qw(WiKicker::CGI::View);
}

sub view {
  my ($self, $request, $response, $begin_time) = @_;

  $self->{begin_time} = $begin_time;

  $self->_setup($request, $response);
  $self->set_html_http_header($response);
  $response->flush;
  $self->make_start_html($request, $response);
  $response->flush;
  $self->make_html_header($request, $response);
  $response->flush;
  $self->make_html_body($request, $response);
  $response->flush;
  $self->make_html_footer($request, $response);
}

sub _setup {
  my ($self, $request, $response);
}

# override subclass
# For example, return '503 Unavailable';
sub http_status {
  return '';
}

sub set_html_http_header {
  my ($self, $response) = @_;

  my %param;
  $param{-type} = 'text/html; charset=UTF-8';
  $param{-charset} = 'UTF-8';
  if (@{$response->{cookies}} > 0) {
    $param{-cookie} = $response->{cookies};
  }
  my $last_modified = $self->request->attribute('http_last_modified_for_send');
  if ($last_modified) {
    $param{-Last_Modified} = WiKicker::Time::gmtime_to_RFC1123($last_modified);
  }
  my $status = $self->http_status;
  if ($status ne '') {
    $param{-status} = $status;
  }
  
  $response->http_header($self->request->query->header(%param));
}


sub make_start_html {
  my ($self, $request, $response) = @_;

  my $run_count = $request->attribute('run_count') || 0;
  my %param;
  $param{-head} = [ CGI::meta({-http_equiv => 'Content-Type',
                               -content => 'text/html;charset=UTF-8'}),
                    "<!-- WiKicker $WiKicker::VERSION on $^O Perl $] -->",
                    "<!-- Run: $$($run_count) -->",
                  ];
  my @link_element_list = $self->make_link_element_list;
  if (@link_element_list) {
    push(@{$param{-head}}, @link_element_list);
  }
  $param{-title} = $self->make_html_head_title;
  $param{-lang} = $request->lang;
  my @stylesheet_uri_list = $self->stylesheet_uri_list;
  if (@stylesheet_uri_list) {
    #$param{-style} = { -src => [ @stylesheet_uri_list ] };
    # for old CGI.pm
    map { push(@{$param{-head}},
               CGI::Link({-rel => 'stylesheet',
                          -type => 'text/css',
                          -href => $_})) } @stylesheet_uri_list;
  }
  $param{-dtd} = '-//W3C//DTD HTML 4.01//EN';

  my $head_comments = $self->request->attribute('head_comments');
  if (defined $head_comments) {
    map { push(@{$param{-head}}, $self->request->query->comment($_)) }
      @$head_comments;
  }
  
  $response->add_content($self->request->query->start_html(%param));
}

sub make_html_header {
  my ($self, $request, $response) = @_;
}

sub make_html_body {
  my ($self, $request, $response) = @_;
}

sub make_html_footer {
  my ($self, $request, $response) = @_;

  $response->add_content('</body></html>');
}

sub make_link_element_list { return () };
sub make_html_head_title { '(title)' }
sub stylesheet_uri { '' }

1;
