#
# WiKicker
#
#  Makefile.PL
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Configuration.pm,v 1.13 2004/01/24 15:37:31 naney Exp $
#
#

package WiKicker::Configuration;

use 5.005;
use strict;
use Carp;
use WiKicker::WRI::Factory;

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  return $self;
};

sub properties {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{properties} = $value;
  }
  return $self->{properties};
}

sub database_directory {
  my $self = shift;

  if ($self->properties->get('wikicker.database.directory')) {
    return $self->properties->get('wikicker.database.directory');
  }
  else {
    return $self->wikicker_home . '/var/database';
  }
}

sub wikicker_home {
  croak "You must override WiKicker::Configuration::wikicker_home";
}

sub automatic_link_enabled {
  my ($self, $load_average) = @_;

  croak "no load average object" unless defined $load_average;

  my $setting = $self->properties->get('automaticlink') || '';
  unless ($setting) {
    return 1;
  }

  if ($setting =~ /^\s*enable\s*$/) {
    return 1;
  }

  if ($setting =~ /^\s*load\s*<\s*(\d(?:\.\d+)?)\s*$/) {
    my $load = $1;
    if ($load <= 0) {
      return 0;
    }
    my $load_average_1 = $load_average->load_average_1;
    if ($load_average_1 < 0) {
      return 0;
    }
    if ($load_average_1 < $load) {
      return 1;
    }
    return 0;
  }

  return 0;
}

sub create_document_parser {
  my $self = shift;

  eval "use WiKicker::WikiPage::DocumentParser";
  if ($@) {
    croak "Can't load WiKicker::WikiPage::DocumentParser: $@";
  }
  return WiKicker::WikiPage::DocumentParser->new(wri_factory => $self->create_wri_factory);
}

sub create_wri_factory {
  my $self = shift;

  return WiKicker::WRI::Factory->new(user_rwrn_map_object => $self);
}

sub user_scheme_to_class_name {
  my ($self, $scheme) = @_;

 return $self->properties->get("RWRN.$scheme");
}

sub last_modified {
  my $self = shift;

  return $self->properties->last_modified;
}

1;
