#
# WiKicker
#
#  PageName.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PageName.pm,v 1.7 2003/11/01 02:57:30 naney Exp $
#
#

package WiKicker::PageName;

use 5.005;
use strict;

use vars qw($no_armor_pattern);

BEGIN {
  $no_armor_pattern = '\b([A-Z][a-z]+([A-Z][a-z]+)+)\b';
}

sub armor_name {
  my $name = shift;

  $name =~ s/:/\\:/g;

  if ($name =~ /^$no_armor_pattern$/o) {
    return $name;
  } else {
    return "[[$name]]";
  }
}

sub normalize {
  my $name = shift;
  $name =~ s/\s+/ /g;
  $name =~ s/^\s+//;
  $name =~ s/\s+$//;

  return $name;
}

sub main_name {
  my $name = shift;

  if ($name =~ m|/([^/]+)$|) {
    return $1;
  }
  return $name;
}  

sub full_page_name_list {
  my ($name, $page_name_list) = @_;

  my @result = ();
  my $target_name;
#   foreach $target_name (@$page_name_list) {
#     if ($target_name =~ m|/\Q$name\E$|) {
#       push(@result, $target_name);
#     }
#   }
  my $substring = '/' . $name;
  my $pattern_length = length($substring);
  foreach $target_name (@$page_name_list) {
    my $pos = rindex($target_name, $substring);
    if ($pos != -1 && (length($target_name) - $pattern_length) == $pos) {
      push(@result, $target_name);
    }
  }

  if (@result) {
    return @result;
  }

  my @segment = split('/', $name);
  if (@segment < 2) {
    return ();
  }

  my $re = join('(/.+)?/', map {"\Q$_\E"} @segment);
  $re = '^(.+/)?' . $re;
  $re .= '$';

  foreach $target_name (@$page_name_list) {
    if ($target_name =~ m|$re|) {
      push(@result, $target_name);
    }
  }
  return @result;
}

1;
