#
# WiKicker
#
#  Controller.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Controller.pm,v 1.5 2004/01/20 10:40:06 naney Exp $
#
#

package WiKicker::WikiCGI::Controller;

use 5.005;
use strict;
use WiKicker::WikiCGI::Configuration;
use WiKicker::WikiCGI::Request;
use WiKicker::WikiCGI::PageFactory;
use WiKicker::WikiCGI::Environment;
use WiKicker::URI;
use WiKicker::CGI::AbstractController;

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::CGI::AbstractController);
}

sub _get_configuration {
  my ($self, $query) = @_;

  my %options = (query => $query);
  if (exists $self->{options}->{properties_file}) {
    $options{properties_file} = $self->{options}->{properties_file};
  }

  return WiKicker::WikiCGI::Configuration->new(%options);
}

sub _get_environment {
  my ($self, $configuration) = @_;

  return WiKicker::WikiCGI::Environment->new(configuration => $configuration);
}

sub _get_request {
  my ($self, $configuration, $query) = @_;

  return WiKicker::WikiCGI::Request->new(configuration => $configuration,
                                         query => $query);
}

sub _run_hook {
  my ($self, $request) = @_;

  return 1 if $self->_request_hook($request);
  return 1 if $self->_null_name_hook($request);
  return 0;
}

sub _create_page {
  my ($self, $request) = @_;

  return WiKicker::WikiCGI::PageFactory->new()->create($request);
}

sub _request_hook {
  my ($self, $request) = @_;

  my $name;
  if ($request->query->path_info eq '/wiki.cgi') {
    if ($request->query->keywords) {
      $name = join('', $request->query->keywords);
    }
    else {
      $name = $request->query->param('mypage');
    }

    if ($name) {
      print $request->query->redirect(WiKicker::URI::make_read_page_url($request->url, $name));
    }
    else {
      print $request->query->redirect($request->url);
    }
    return 1;
  }

  return 0;
}

sub _null_name_hook {
  my ($self, $request) = @_;

  if ($request->name eq '') {
    print $request->query->redirect($request->url);
    return 1;
  }

  return 0;
}

sub _log {
  my ($self, $request) = @_;

  $request->environment->add_recent_log($request);
}

1;

