#
# WiKicker
#
#  HTML.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HTML.pm,v 1.11 2004/01/29 14:39:24 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML;

use 5.005;
use strict;
use WiKicker;
use WiKicker::CGI::View::HTML;
use WiKicker::WikiCGI::View;
use WiKicker::HTML;
use WiKicker::HTML::LinkMaker;
use WiKicker::URI;
use WiKicker::PageName;
use WiKicker::WRI::Resolver;
use WiKicker::Time;
use WiKicker::WikiPage::HtmlFragmentVisitor;
use WiKicker::WikiPage::tDiaryFragmentVisitor;
use WiKicker::WikiPage::Env;
use WiKicker::WikiPage::HtmlTocVisitor;
use CGI qw/-no_xhtml/;
use Time::HiRes qw(gettimeofday tv_interval);
use URI;
use AutoLoader 'AUTOLOAD';

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::CGI::View::HTML WiKicker::WikiCGI::View);
}

sub link_maker {
  my $self = shift;

  if ($self->{link_maker}) {
    return $self->{link_maker};
  }
  else {
    my $link_maker = $self->{link_maker} = WiKicker::HTML::LinkMaker->new;

    $link_maker->script_url($self->request->url);
    $link_maker->db($self->db);
    $link_maker->configuration($self->request->configuration);
    if ($self->request->user) {
      $link_maker->timezone($self->request->user->timezone || 'GMT');
    }
    else {
      $link_maker->timezone('GMT');
    }
  }
  return $self->{link_maker}
}

sub make_link_element_list {
  my $self = shift;

  my $name = $self->request->name;

  my @list;

  push(@list, CGI::Link({-rel => 'start', -href => $self->front_page_url}));
  push(@list, CGI::Link({-rel => 'index', -href => $self->index_page_url}));
  push(@list, CGI::Link({-rel => 'search', -href =>
                         $self->request->url . '/SearchPage.html'}));

  my @ancestors = $self->db->ancestors_page_name_list($name);
  if (@ancestors) {
    push(@list, CGI::Link({-rel => 'up', -href =>
                           WiKicker::URI::make_read_page_url($self->request->url, $ancestors[-1])}));
  }
  
  return @list;
}

sub _get_properties {
  my ($self, $key) = @_;

  $self->request->configuration->properties->get($key);
}

sub make_html_head_title {
  my $self = shift;

  my $name = $self->request->name;

  my $db = $self->db;
  my $subject = '';
  if ($db->exists_live_page($name)) {
    $subject = ' - ' . $db->page_subject($name);
  }
  
  my $prefix = $self->_get_properties('wikicker.name');
  return WiKicker::HTML::escape("$prefix: $name$subject");
}  

sub stylesheet_uri_list {
  my $self = shift;

  my $stylesheet = $self->_get_properties('html.stylesheet');

  unless ($stylesheet) {
    return ();
  }

  my @css_list = split(/\s+/, $stylesheet);

  my @result = map {URI->new_abs($_, $self->request->url) } @css_list;
  return @result;
}

sub stylesheet_type {
  my $self = shift;

  if ($self->{stylesheet_type}) {
    return $self->{stylesheet_type};
  }
  my $stylesheet_type = $self->_get_properties('html.stylesheet.type');
  if (defined $stylesheet_type && $stylesheet_type ne '') {
    $self->{stylesheet_type} = $stylesheet_type;
  }
  else {
    $self->{stylesheet_type} = 'old';
  }

  return $self->{stylesheet_type};
}

sub make_html_header {
  my ($self, $request, $response) = @_;

  my $name = $self->request->name;

  my $result = '';

  if ($self->stylesheet_type eq 'tDiary') {
    $result .= '<div class="main">';
  }
  
  my $media = $self->request->param('media') || '';
  unless ($media eq 'print') {
    $result .= $self->make_header_toolbar($name);
  }
  $result .= $self->make_page_name_div;
  unless ($media eq 'print') {
    $result .= $self->make_page_toolbar($name);
  }

  $response->add_content($result);
}

sub _make_page_name_div_action_prefix {
  my $self = shift;
  
  my $action = $self->request->action;
  if ($self->request->attribute('action_label')) {
    $action = $self->request->attribute('action_label');
  }
  if ($action eq 'read') {
    $action = '';
  }
  else {
    $action = "$action: ";
  }
}

sub make_page_name_div {
  my $self = shift;

  my $result;
  $result .= qq(<div class="head-name">\n);
  if ($self->db->ancestors_page_name_list($self->request->name)) {
    $result .= $self->_make_page_name_div_navigation($self->request->name);
    $result .= $self->_make_page_name_div_title(WiKicker::PageName::main_name($self->request->name));
  }
  else {
    $result .= $self->_make_page_name_div_title($self->request->name);
  }
  if ($self->request->attribute('revision')) {
    $result .= q(This is old revision. ->);
    $result .= qq(<a href=") . $self->latest_url . qq(">latest</a>);
  }
  $result .= $self->make_header_last_modified;
  $result .= qq(\n</div>\n);

  return $result;
}

sub _head_hierarchy_class {
  my $self = shift;

  if ($self->stylesheet_type eq 'tDiary') {
    return 'adminmenu';
  }

  return 'head-hierarchy';
}

sub _make_page_name_div_navigation {
  my ($self, $name) = @_;

  my @path = split('/', $name);
  if (@path <= 1) {
    return '';
  }

  my @path_name;
  my @path_link;
  for (my $i = 0; $i < @path - 1; $i++) {
    push(@path_name, $path[$i]);
    push(@path_link,
         $self->link_maker->make_page_name_link(join('/', @path_name),
                                                '',
                                                $path[$i], 0));
  }    
  
  my $search_url = $self->request->url . '?search=' .
    WiKicker::URI::escape_URI($name);

  my $class = $self->_head_hierarchy_class;

  my $result;
  $result .= qq(<div class="$class">[ );
  $result .= join(' &gt; ', @path_link);
  $result .= ' ] ';
  $result .= $self->_make_anchor($search_url, 'Search: ' . $name, $name);
  $result .= qq(</div>\n);

  return $result;
}

sub _make_anchor {
  my ($self, $url, $title, $content) = @_;

  $title   = WiKicker::HTML::escape_attribute($title);
  $content = WiKicker::HTML::escape($content);

  return qq(<a href="$url" title="$title">$content</a>);
}

sub _make_page_name_div_title {
  my ($self, $short_name) = @_;

  my $action = $self->_make_page_name_div_action_prefix;

  my $search_url
    = $self->request->url . '?search=' .
      WiKicker::URI::escape_URI($short_name);
  my $result;
  $result .= q(<h1>);
  $result .= WiKicker::HTML::escape($action);
  $result .= $self->_make_anchor($search_url, 'Search: ' . $short_name, $short_name);
  if ($self->request->attribute('revision')) {
    $result .= " (";
    $result .= WiKicker::HTML::escape($self->request->attribute('revision'));
    $result .= ")";
  }
  $result .= q(</h1>);
  return $result;
}  

sub _header_last_modified_class {
  my $self = shift;

  if ($self->stylesheet_type eq 'tDiary') {
    return 'adminmenu';
  }

  return 'head-lastmodified';
}

sub make_header_last_modified {
  my $self = shift;

  my $db = $self->db;
  return '' unless ($self->enable_header_last_modified);

  my $result = '';
  my $last_modified = $db->last_modified($self->request->name);
  my $last_modified_string;
  if ($last_modified) {
    $last_modified_string
      = WiKicker::Time::japanese_date_html_str($last_modified,
                                               $self->timezone);
  }
  else {
    $last_modified_string = 'unknown';
  }
  
  my $revision = $self->request->attribute('latestrevision');
  my $author = $db->page_author($self->request->name);
  my $class = $self->_header_last_modified_class;
  $result .= qq(<div class="$class">\n);
  my $last_modified_text = $self->request->maketext('Last-Modified');
  $result .= qq($last_modified_text: $last_modified_string);
  if ($revision) {
    $result .= qq(; Revision: $revision);
  }
  if ($author) {
    $author = $self->link_maker->make_page_name_link($author, '', '', 0);
    $result .= qq(; by $author);
  }
  $result .= q(</div>);
  return $result;
}

sub _wrap_toolbar_item {
  my ($self, $text) = @_;

  if ($self->stylesheet_type eq 'tDiary') {
    return qq(<span class="adminmenu">$text</span>);
  }
  else {
    return $text;
  }
}

sub _make_header_toolbar_item {
  my ($self, $url, $text, $title) = @_;

  my $link_text = WiKicker::HTML::escape($self->request->maketext($text));
  if ($title ne '') {
    my $link_title = WiKicker::HTML::escape_attribute($self->request->maketext($title));
    return $self->_wrap_toolbar_item(qq(<a href="$url" title="$link_title">$link_text</a>));
  }
  else {
    return $self->_wrap_toolbar_item(qq(<a href="$url">$link_text</a>));
  }
}

sub make_header_toolbar {
  my ($self, $name) = @_;

  my $div_class;
  if ($self->stylesheet_type eq 'tDiary') {
    $div_class = "adminmenu";
  }
  else {
    $div_class = "head-toolbar";
  }
  
  my $result = qq(\n<!-- begin header toolbar -->\n<div class="$div_class">\n);
  my $query = $self->request->query;
  $result .= $query->start_form(-action => $self->request->url, -method=> 'POST');
  $result .= qq(  <div>\n   );
  
  my $url = $self->request->url;

  my @toolbar_item;
  push(@toolbar_item,
       $self->_make_header_toolbar_item($url, 'FrontPage', 'Go top page'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/RecentChanges.html",
                                        'changes',
                                        'recent changes'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item($self->index_page_url,
                                        'index',
                                        'page index'));
  if ($self->enable_header_toolbar_create) {
    push(@toolbar_item,
         $self->_make_header_toolbar_item("$url/CreatePage.html",
                                          'create',
                                          'create new page'));
  }
       
  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/SearchPage.html",
                                        'search_button',
                                        'search'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/PreferencePage.html",
                                        'preference_button',
                                        'preference'));
  
  my $user_name = $self->request->user->name;
  if ($user_name && $self->db->exists_live_page($user_name)) {
    push(@toolbar_item,
         $self->_wrap_toolbar_item($self->link_maker
                                   ->make_page_name_link($user_name,
                                                         '',
                                                         '', 0)));
  }
  $result .= join(qq(\n | ), @toolbar_item);
  $result .= qq(\n | );

  $result .= $query->textfield(-name => 'name',
                               -title => 'Input PageName or search query',
                               -size => 14);
  $result .= $query->submit(-name => 'action', -value => 'read',
                            -title => 'Go to the page');
  $result .= $query->submit(-name => 'action', -value => 'search',
                            -title => 'Search the page');
  $result .= qq(\n  </div>);
  $result .= $query->end_form;
  
  $result .= qq(\n</div><!-- end $div_class -->\n<!-- end header toolbar -->\n);

  return $result;
}

sub make_page_toolbar {
  my ($self, $name) = @_;

  my @link;
  push(@link, $self->make_page_toolbar_edit($name));
  push(@link, $self->make_page_toolbar_copy($name));
  push(@link, $self->make_page_toolbar_diff($name));
  push(@link, $self->make_page_toolbar_latest($name));
  push(@link, $self->make_page_toolbar_rlog($name));
  push(@link, $self->make_page_toolbar_raw($name));

  my $link_string;

  if ($self->stylesheet_type eq 'tDiary') {
    $link_string = join("\n| ", map { $self->_wrap_toolbar_item($_) } grep { $_ ne '' } @link);
  }
  else {
    $link_string = join("\n| ", grep { $_ ne '' } @link);
  }

  return '' unless $link_string;

  if ($self->stylesheet_type eq 'tDiary') {
    return qq(<div class="adminmenu">\n$link_string</div><!-- end adminmenu -->\n);
  }
  else {
    return qq(<div class="page-toolbar">\n$link_string</div>\n);
  }
}  

sub make_page_toolbar_edit {
  my ($self, $name) = @_;
  if ($self->enable_header_toolbar_edit) {
    return $self->_make_localized_link($self->edit_url,
                                       'edit',
                                       'edit this page');
  }
  return '';
}

sub make_page_toolbar_copy {
  my ($self, $name) = @_;
  
  if ($self->enable_header_toolbar_copy) {
    return $self->_make_localized_link($self->copy_url,
                                       'copy',
                                       'copy this page');
  }
  return '';
}

sub make_page_toolbar_diff {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_diff) {
    return $self->_make_localized_link($self->diff_url,
                                       'diff',
                                       'show difference');
  }
  else {
    return '';
  }
}
  
sub make_page_toolbar_latest {
  my ($self, $name) = @_;

  if ($self->request->attribute('revision')) {
    my $latest_url = $self->latest_url;
    return qq(<a href="$latest_url">latest</a>);
  }
  else {
    return '';
  }
}
    
sub make_page_toolbar_rlog {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_rlog) {
    my $rlog_url = $self->rlog_url;
    return qq(<a href="$rlog_url">rlog</a>);
  }
  else {
    return '';
  }
}

sub make_page_toolbar_raw {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_raw) {
    my $raw_url = $self->raw_url;
    return qq(<a href="$raw_url" title="source of this WikiPage">raw</a>);
  }
  else {
    return '';
  }
}

# sub track_back_ping_url {
#   my $self = shift;

#   if ($self->request->attribute('normalpage')
#       && $self->request->action eq 'read') {
#     return WiKicker::URI::make_track_back_url($self->request->query->url,
#                                               $self->request->name);
#   }
#   else {
#     '';
#   }
# }

# sub make_track_back_div {
#   my $self = shift;

#   my $entry_map = $self->db->cache('trackback', $self->request->name);
#   unless (defined $entry_map) {
#     return '';
#   }
#   $entry_map = Storable::thaw($entry_map);
#   my @entries = $entry_map->get_entries_list;

#   my $result = '<h2>TrackBack</h2>';
#   $result .= '<div class="trackback">';
#   if (@entries) {
#     $result .= '<dl>';
#     for my $entry (@entries) {
#       my $escaped_url = WiKicker::HTML::escape($entry->url);
#       my $href_url = WiKicker::HTML::escape_attribute($entry->url);
#       my $title = $entry->title;
#       if ($title) {
#         $title = WiKicker::HTML::escape($title);
#       }
#       else {
#         $title = $escaped_url;
#       }
      
#       $result .= '<dt>';
#       unless ($href_url =~ /^(java-script|vbscript|about):/i) {
#         $result .= qq(<a href="$href_url">$title</a>);
#       }
#       else {
#         $result .= qq($title);
#       }
#       $result .= '</dt>';
#       if ($entry->excerpt) {
#         $result .= q(<dd>excerpt: );
#         $result .= WiKicker::HTML::escape($entry->excerpt);
#         $result .= q(</dd>);
#       }
#       if ($entry->blog_name) {
#         $result .= q(<dd>blog_name: );
#         $result .= WiKicker::HTML::escape($entry->blog_name);
#         $result .= q(</dd>);
#       }
#       if ($entry->title) {
#         $result .= qq(<dd>url: <a href="$href_url">$escaped_url</a></dd>);
#       }
#       $result .= '<dd>time: ';
#       $result .=  WiKicker::Time::japanese_date_html_str($entry->timestamp,
#                                                          $self->timezone);
#       $result .= '</dd>';

#     }
#     $result .= '</dl></div>';
#   }
#   return $result;
# }

sub make_html_footer {
  my ($self, $request, $response) = @_;

  my $result = $self->_make_html_footer_from_text;

  if ($self->stylesheet_type eq 'tDiary') {
    $result .= '</div>'; #div.main
  }
  
  $result .= '</body></html>';

  $response->add_content($result);
}

sub _make_html_footer_from_text {
  my $self = shift;

  my $load_average = $self->request->load_average->load_average_string;
  my $process_time = "Process Time: " . tv_interval($self->{begin_time}) . "s";
  my $short_name = $self->request->name;
  $short_name =~ s|^(.*)/||;
  my $short_name_attribute = WiKicker::HTML::escape_attribute($short_name);


  my $footer = $self->_load_footer_file;
  $footer =~ s/\[\[value:LoadAverage\]\]/$load_average/g;
  $footer =~ s/\[\[value:ProcessTime\]\]/$process_time/g;
  $footer =~ s/\[\[value:ShortNameAttribute\]\]/$short_name_attribute/g;
  $footer =~ s/\[\[value:Version\]\]/$WiKicker::VERSION/g;

  return $footer;
}

sub _load_footer_file {
  my $self = shift;
  
  my @footer_files = map {$_ . '/WiKicker/Template/HTML/footer.txt' } @INC;

  my $user_footer_file = $self->_get_properties('html.footer.file');
  if (defined $user_footer_file && $user_footer_file ne '') {
    unshift(@footer_files, $user_footer_file);
  }

  my $footer = '';
  for my $path (@footer_files) {
    if (-e $path) {
      unless (open(FOOTER, $path)) {
        warn "Can't open footer: $path: $!";
        next;
      }
      else {
        local $/;
        $footer = <FOOTER>;
        close(FOOTER);
        last;
      }
    }
  }
  return $footer;
}

# sub _make_html_footer_by_template {
#   my $self = shift;

#   my $load_average_result = `uptime`;
#   $load_average_result =~ /(load.*)$/;
#   my $load_average = $1 || '';
#   my $process_time = "Process Time: " . tv_interval($self->{begin_time}) . "s";
#   my $short_name = $self->request->name;
#   $short_name =~ s|^(.*)/||;
#   my $short_name_attribute = WiKicker::HTML::escape_attribute($short_name);

#   my @include = map {$_ . '/WiKicker/Template' } @INC;

#   my $config = { INCLUDE_PATH => [ @include ] };
#   WiKicker::Loader::load_module("Template");
#   my $template = Template->new($config);
#   my $vars = { load_average => $load_average,
#                process_time => $process_time,
#                short_name_attribute => $short_name_attribute,
#                version => $WiKicker::VERSION,
#              };
#   my $template_out = '';
#   $template->process('HTML/footer.tt2', $vars, \$template_out);
#   return $template_out;
# }

# sub make_debug_info {
#   my $self = shift;
#   my $result = '';
#   my $query = $self->request->query;
#   $result .= '$query->url:' . $self->request->query->url;
#   $result .= $query->br;
#   $result .= 'PATH_INFO:'. $query->path_info();
#   $result .= $query->br;
#   $result .= 'QUERY_STRING:' . $query->query_string();
#   $result .= $query->br;
#   $result .= 'name:' . $self->request->name;
#   $result .= $query->br;
#   $result .= $query->br;
#   my $key;
#   for $key (sort keys %ENV) {
#     $result .= $query->br;
#     $result .= "$key = " . $ENV{$key};
#   }
#   return $result;
# }
  


sub enable_header_toolbar_edit {
  return 0;
}

sub enable_header_toolbar_copy {
  return 0;
}

sub enable_header_toolbar_create {
  return 0;
}

sub enable_header_toolbar_rlog {
  return 0;
}

sub enable_header_toolbar_raw {
  return 0;
}

sub enable_header_toolbar_diff {
  return 0;
}

sub enable_header_last_modified {
  my $self = shift;

  if ($self->request->attribute('editable')) {
    return 1;
  }
  else {
    return 0;
  }
}

sub edit_url {
  my $self = shift;

  $self->request->url
    . '?edit=' . WiKicker::URI::escape_URI($self->request->name);
}

sub copy_url {
  my $self = shift;

  $self->request->url
    . '?name=CopyPage&amp;from='
      . WiKicker::URI::escape_URI($self->request->name);
}

sub rlog_url {
  my $self = shift;

  $self->request->url .
    '?rlog=' . WiKicker::URI::escape_URI($self->request->name);
}

sub diff_url {
  my $self = shift;

  $self->request->url .
    '?diff=' . WiKicker::URI::escape_URI($self->request->name);
}

sub raw_url {
  my $self = shift;

  my $revision_param = '';
  if ($self->request->attribute('revision')) {
    $revision_param = '&revision=';
    $revision_param .= WiKicker::URI::escape_URI($self->request->attribute('revision'));
  }
  return $self->request->url . '?name=' .
    WiKicker::URI::escape_URI($self->request->name) . '&amp;format=raw'
        . $revision_param;
}

sub latest_url {
  my $self = shift;

  $self->request->url .'?name=' .
    WiKicker::URI::escape_URI($self->request->name);
}

sub timezone {
  my $self = shift;

  return $self->request->user->timezone
    || $self->_get_properties('user.timezone');
}

sub front_page_url {
  my $self = shift;

  return $self->request->url;
}

sub index_page_url {
  my $self = shift;

  return $self->request->url . '/IndexPage.html';
}

sub _make_localized_link {
  my ($self, $url, $text, $title) = @_;

  my $L10N_title
    = WiKicker::HTML::escape_attribute($self->request->maketext($title));
  my $L10N_text
    = WiKicker::HTML::escape($self->request->maketext($text));

  return qq(<a href="$url" title="$L10N_title">$L10N_text</a>);
}

sub make_document_node;
sub make_html_fragment;
sub make_html_fragment_with_toc;
sub make_html_fragment_from_document_node;
sub make_toc_from_document_node;

1;

__END__

sub make_document_node {
  my ($self, $wiki_page) = @_;

  return
    $self->request->configuration->create_document_parser->parse($wiki_page);
}

sub make_html_fragment {
  my ($self, $wiki_page, $no_edit_section, $no_automatic_link) = @_;

  my $document_node = $self->make_document_node($wiki_page);
  unless ($document_node) {
    return q(<p>Sorry, page parse error.</p>);
  }
  return $self->make_html_fragment_from_document_node($document_node, 0, $no_edit_section, $no_automatic_link);
}


sub make_html_fragment_with_toc {
  my ($self, $wiki_page, $flush) = @_;

  my $document_node = $self->make_document_node($wiki_page);
  unless ($document_node) {
    return q(<p>Sorry, page parse error.</p>);
  }
  my $toc = $self->make_toc_from_document_node($document_node);
  if ($flush) {
    print $toc;
    $toc = '';
  }
  return $toc . $self->make_html_fragment_from_document_node($document_node, $flush, 0, ! $self->request->configuration->automatic_link_enabled($self->request->load_average));
}


sub make_html_fragment_from_document_node {
  my ($self, $document_node, $flush, $no_edit_section, $no_automatic_link) = @_;
  my $visitor;
  if ($self->stylesheet_type eq 'tDiary') {
    $visitor = WiKicker::WikiPage::tDiaryFragmentVisitor->new;
  }
  else {
    $visitor = WiKicker::WikiPage::HtmlFragmentVisitor->new;
  }

  my $env = WiKicker::WikiPage::HtmlEnv
    ->new(user => $self->request->user,
          resolver => $self->request->environment->resolver,
          wikicker_url => $self->request->url,
          wri_factory => $self->request->configuration->create_wri_factory,
          configuration => $self->request->configuration,
          environment => $self->request->environment,
          query => $self->request->query,
          page_name => $self->request->name);

  $visitor->env($env);
  $visitor->no_edit_section($no_edit_section);
  $visitor->no_automatic_link($no_automatic_link);
  if ($flush) {
    $visitor->flush_closure(sub { print shift; });
  }

  $visitor->visit($document_node);
  #warn "Make Wiki HTML Time: " . tv_interval($start_time) . "s";

  #IFDEBUG: WiKicker::total_size("HtmlFragmentVisitor", $visitor);
  return $visitor->content;
}

sub make_toc_from_document_node {
  my ($self, $document_node) = @_;

  my $visitor = WiKicker::WikiPage::HtmlTocVisitor->new;
  $visitor->visit($document_node);
  if ($visitor->size <= 1) {
    return '';
  }

  return qq(<div class="toc">\n) . $visitor->content . qq(</div>\n);
}
