#
# WiKicker
#
#  Recent.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Recent.pm,v 1.3 2003/12/29 12:35:58 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML::Recent;

use 5.005;
use strict;

use WiKicker::WikiCGI::View::HTML;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::View::HTML);

sub make_html_body {
  my ($self, $request, $response) = @_;

  my $db = $self->db;

  my $sub = $self->request->configuration->properties->get('param.RecentChanges.UrlFilter');
  if (defined $sub && $sub =~ /^\s*sub\s*{/) {
    my $closure = eval $sub;
    if ($@) {
      warn $@;
    }
    else {
      $self->{recent_url_filter} = $closure;
    }
  }
    
  $response->add_content(qq(<ol class="changes">\n));
  for my $record (@{$request->attribute('list')}) {
    $self->_make_line($response, $db,
                      $record->[0], # name
                      $record->[1], # date
                      $record->[2] || '', # author
                      $record->[3] #subject
                     );
  }

  $response->add_content(qq(</ol>\n));
}

sub _make_line {
  my ($self, $response, $db, $name, $date, $author, $subject) = @_;

  my $date_str = WiKicker::Time::japanese_date_html_str($date,
                                           $self->request->user->timezone);

  my $name_content = WiKicker::HTML::escape($name);
  my $name_str;
  if ($db->exists_live_page($name)) {
    my $href = $self->link_maker->make_page_read_href($name);
    if (exists $self->{recent_url_filter}) {
      my $new_href = '';
      eval {
        $new_href = &{$self->{recent_url_filter}}(url => $href, last_modified => $date);
      };
      if ($@) {
        warn $@;
      }
      if ($new_href ne '') {
        $href = $new_href;
      }
    }
    
    $name_str = qq(<a href="$href">$name_content</a>);
  }
  else {
    $name_str = "$name_content [deleted]";
  }
    
  my $author_str = $author ? '(' . $self->link_maker->make_page_name_link($author, '', '', 0) . ')' : '';
  my $subject_str = $subject ? ' - ' . WiKicker::HTML::escape($subject) : '';

  $response->add_content(qq(<li>$date_str $name_str <small>$author_str $subject_str</small></li>\n));
  $response->flush;
}

sub enable_header_toolbar_create {
  return 1;
}

1;
